import websocket
import _thread
import time
import weakref
import queue


master_dict = {}
master_queues = []
master_lock =  _thread.allocate_lock()


def config(field):
    def f(field=field):
        try:
            master_lock.acquire()
            return master_dict.get(field)
        finally:
            master_lock.release()
    return f

def destroy_queue(proxy):
    master_lock.acquire()
    master_queues.remove(proxy)
    master_lock.release()

def get_queue():
    que = queue.Queue()
    master_lock.acquire()
    master_queues.append(weakref.proxy(que, destroy_queue))
    for k, v in master_dict.items():
        que.put((k, v))
    master_lock.release()
    return que

def on_message(ws, message):
    try:
        master_lock.acquire()
        if '\t' in message:
            k, v = message.split('\t', 1)
            master_dict[k] = v
            for queue in master_queues:
                queue.put((k, v))
        else:
            for queue in master_queues:
                queue.put((message, None))
            del(master_dict[message])
    finally:
        master_lock.release()

        
def on_error(ws, error):
    print("ERROR:", error)
    pass

def on_close(ws, close_status_code, close_msg):
    pass

def on_open(ws):
    ws.send(ws.key)

def startup(key, url="wss://stream.realtimeconfig.com:1443/"):
    ws = websocket.WebSocketApp(url,
                                on_open=on_open,
                                on_message=on_message,
                                on_error=on_error,
                                on_close=on_close)
    ws.key = key
    
    _thread.start_new_thread(ws.run_forever, ())






import sys
from realtimeconfig import get_queue, startup
import argparse


def main():
    namespace = parser.parse_args(sys.argv[1:])
    startup(namespace.key, namespace.url)
    
    queue = get_queue()

    while True:
        k, v = queue.get()
        print(k, v, sep="\t")


parser = argparse.ArgumentParser(description='realtimeconfig-monitor')
parser.add_argument('--key',
                    required=True,
                    help='Your realtimeconfig API key')
parser.add_argument('--url',
                    default='wss://stream.realtimeconfig.com:1443/',
                    help='The realtimeconfig source.  Change this if you are using a proxy')


if __name__ == "__main__":
    main()



import os.path
from setuptools import setup
from unittest import TestLoader


def tests():
    return TestLoader().discover('test', pattern='test_*.py')


setup(
    name='realtimeconfig',
    version='0.0.1',
    packages=[
        "realtimeconfig/"
    ],
    url='https://realtimeconfig.com',
    install_requires=[
        "websocket-client",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'realtimeconfig-monitor=realtimeconfig.monitor:main',
            
    ]})
